// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_POSIX_ACCEPTOR_H
#define HU_VARADIISTVAN_SCPL_IO_POSIX_ACCEPTOR_H
//------------------------------------------------------------------------------

#include "../Failable.h"

#include "ServerSocket.h"

#include <cstdlib>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * Class representing the accepting of an incoming connection on a
 * socket.
 */
class Acceptor : public FailableReference<Acceptor>
{
protected:
    /**
     * The socket we belong to.
     */
    ServerSocket& socket;

    /**
     * The file descriptor accepted the last time.
     */
    int acceptedFD;

    /**
     * The buffer for the remote address, if we are interested.
     */
    unsigned char* address;

    /**
     * The length of the remote address buffer.
     */
    size_t addressLength;

protected:
    /**
     * Construct the acceptor.
     *
     * @param addressLength the length of the buffer for the peer
     * address. If 0, not buffer will be created.
     */
    Acceptor(ServerSocket* socket, size_t addressLength = 0);

public:
    /**
     * Destroy the acceptor.
     */
    virtual ~Acceptor();

    /**
     * Start accepting a connection.
     *
     * If a connection is already accepted, but not retrieved yet, it
     * returns true.
     *
     * If a new socket becomes immediately available, it returns true.
     *
     * If some error occurs, the acceptor is marked as failed.
     */
    bool accept();

protected:
    /**
     * Called when a connection gets processed and we can become ready
     * for the next one.
     */
    void connectionProcessed();

private:
    /**
     * Get the failable object.
     */
    const Failable& getFailable() const;

    /**
     * Get the failable object.
     */
    Failable& getFailable();

    friend class FailableReference<Acceptor>;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline Acceptor::Acceptor(ServerSocket* socket, size_t addressLength) :
    socket(*socket),
    acceptedFD(-1),
    address( (addressLength==0) ? 0 : new unsigned char[addressLength]),
    addressLength(addressLength)
{
}

//------------------------------------------------------------------------------

inline void Acceptor::connectionProcessed()
{
    acceptedFD = -1;
}

//------------------------------------------------------------------------------

inline const Failable& Acceptor::getFailable() const
{
    return socket;
}

//------------------------------------------------------------------------------

inline Failable& Acceptor::getFailable()
{
    return socket;
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_POSIX_ACCEPTOR_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
